import {AfterViewInit, Component, ElementRef, ViewChild} from '@angular/core';
import PhylocanvasGL, {TreeTypes} from '@phylocanvas/phylocanvas.gl';
import {Router} from '@angular/router';
import {DataService} from 'src/app/services/data.service';
import {DialogComponent} from '../dialog/dialog.component';
import {forkJoin, map, take} from 'rxjs';
import {MatDialog} from "@angular/material/dialog";
import {MatSnackBar} from "@angular/material/snack-bar";

@Component({
  selector: 'slf-families-tree',
  templateUrl: './families-tree.component.html',
  styleUrls: ['./families-tree.component.scss']
})
export class FamiliesTreeComponent implements AfterViewInit {
  @ViewChild('tree') treeDiv!: ElementRef<HTMLDivElement>;
  private tree: PhylocanvasGL;
  public databaseDescription?: string;

  constructor(
    private readonly dialog: MatDialog,
    private readonly router: Router,
    private readonly snackBar: MatSnackBar,
    private readonly dataService: DataService
  ) {
  }


  public ngAfterViewInit(): void {
    forkJoin({
      newick: this.dataService.getNewick().pipe(take(1)),
      style: this.dataService.listFamilyNames()
        .pipe(
          take(1),
          map(familyNames => familyNames.reduce((styles, family) => {
            styles[family] = {fillColour: 'red'};
            return styles;
          }, {}))
        )
    }).subscribe(newickAndStyles => {
      this.renderizeTree(newickAndStyles.newick, newickAndStyles.style);
    });


    this.dataService.getDatabaseInfo()
      .subscribe(info => {
        this.databaseDescription = info.description
      });

  }

  private count_nodes(newickTree: string): number {
    const regex: RegExp = /[^:;,()\s]*(?:\s*:\s*[\d.]+\s*)?([,);])|(\S)/g;
    newickTree += ";";

    return (function recurse(): [string, number] {
      let all: string, delim: string, ch: string;

      // @ts-ignore
      [all, delim, ch] = regex.exec(newickTree);
      let count: number = 1; // Current node
      if (ch == "(") {
        let child_count: number;
        while ("(,".includes(ch)) {
          [ch, child_count] = recurse();
          count += child_count;
        }
        [all, delim, ch] = regex.exec(newickTree);
      }
      return [delim, count];
    })()[1];
  }

  private renderizeTree(newickTree: string, styles: { [key: string]: { fillColour: string } }): void {
    const screenSize: DOMRect = this.treeDiv.nativeElement.getBoundingClientRect();
    const nodeSize: number = 16;
    screenSize.width = screenSize.width * 0.95;
    screenSize.height = 100 + this.count_nodes(newickTree) * nodeSize;

    this.tree = new PhylocanvasGL(
      this.treeDiv.nativeElement,
      {
        interactive: false,
        showLabels: true,
        showLeafLabels: true,
        size: screenSize,
        alignLabels: false,
        nodeSize: nodeSize,
        fillColour: '#3f51b5',
        source: newickTree,
        type: TreeTypes.Rectangular,
        styles: styles,
        styleLeafLabels: true
      }
    );

    this.tree.setProps({
      interactive: true
    });
  }

  public onGoToFamily(): void {
    const param = this.tree['props']['selectedIds'];
    if (param !== undefined) {
      const familyId = param[0];
      let idMultipleValues = familyId.split(/(?=[A-Z])/);
      if (idMultipleValues.length > 1) {
        forkJoin(
          {
            // take(1) is needed as forkJoin requires the observable to complete
            firstValue: this.dataService.hasFamily(idMultipleValues[0]).pipe(take(1)),
            secondValue: this.dataService.hasFamily(idMultipleValues[1]).pipe(take(1))
          }
        ).subscribe(familiesPresence => {
          this.onOpenDialog(idMultipleValues[0], familiesPresence['firstValue'], idMultipleValues[1], familiesPresence['secondValue']);
        });
      } else {
        this.dataService.hasFamily(familyId)
          .subscribe(hasFamily => {
            if (hasFamily) {
              this.router.navigate(['family', familyId]);
            } else {
              this.snackBar.open(`Data not found for family: ${param}`, 'Close', {
                horizontalPosition: 'end',
                verticalPosition: 'bottom',
                duration: 4000
              });
            }
          });
      }
    }
  }

  private onOpenDialog(firstValue: string, existFirstValue: boolean, secondValue: string, existSecondValue: boolean): void {
    const dialogRef = this.dialog.open(DialogComponent, {
      width: '450px',
      data: {
        'firstValue': firstValue,
        'existFirstValue': existFirstValue,
        'secondValue': secondValue,
        'existSecondValue': existSecondValue
      }
    });
  }
}
